%% Author: Lionel Feugère lionel.feugere@riseup.net
% institutions: MIVEGEC - Institut de Recherche pour le Développement, France
%               NRI - University of Greenwhich, UK

%%%%%%%%%%%%%%%% Context %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab code for experiment about hearing threshold sensitivity in male swarming mosquitoes

%%%%%%%%%%%%%%%% Data dependence %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% "_data_tbl2_tstart-0s_tstep-0.04s_tend-27s_21-Janv-2021.mat" (5Go); this file can be computed from data-processing_a.m

%%%%%%%%%%%%%%%% Function dependence %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fctTableOverFactor.m

%%%%%%%%%%%%%%%% What this code does%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Table creation for R analysis: compute 1 response parameter per sound stimulus, then average over the subreplicates to have 1 value per replicate
    % Parameters
        % Choose on which time-window to compute parameters
        % Filtering data: sex, number of swarming mosquitoes, sound stimulus types
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    

clear variables
close all

% Duration before and after stimuli
DurPause_sec=10; 
% Stimulus duration
DurStim_sec=7;    

% These should correspond to preprocessed data. Otherwise, you should run PlaybackExp_analysis_a.m with these parameter-values first
t_start=0; % start time (-th sec)
t_step=0.04; % sampling period
t_end=27; % end time (-th sec) 

disp('Loading data...')
load(strcat('_data_tbl2_tstart-',num2str(t_start),'s_tstep-',num2str(t_step),'s_tend-',num2str(t_end),'s','_20-Jan-2021.mat'));




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Table creation for R analysis

koko_v = {9}; % -th sec from where to start computing parameters (up to 10th s) % {3,9};
for koko_i = 1:length(koko_v)
%%%%%%% Start of user parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
    data_tbl2ter=data_tbl2; % because we are going to modify data_tbl2ter (we dont want to modifiy data_tbl2 whoch is the raw data

    % Filtering data
        sex_choice='m'; % 'f', 'm', or 'both' (flighting mosquito, not the stimuli)
        Nobjects=NaN; % Number of objects swarming at the same time to consider (established by eye during the experiment)
                      % NaN means = all, [1] keeps onlyy 1-swarming mosq, [2] keeps only 2-...., [2 3] keeps ponly 2- and 3-swarming... 
        %%% Time intervals with / without sound stimulus
            t_noStim=single([koko_v{koko_i} 10]); % Min-max time interval (second value not included) for the "without sound stimuli" reference
                                     % We can have [20 27] but it is dangerous as I did not check whether the mosquitoes were still swarming after the stimulus end
            t_stim=single(DurPause_sec+[0 t_noStim(2)-t_noStim(1)]); % Min-max time interval (second value not included) to be analyzed   
        remov_spl_b=0; % remove all the 1b, 2b, 3b, 4b sound level rows (i.e. when the stimuli is a 1-mosquito stimulus)
        remov_gamb=1; % remove the gambiae data (keeps only the coluzzii)
        remov_synth=0; % remove the synthetic stimuli 
        remov_1harm=0; % remove  the stimuli  with only 1 harmonics
            if remov_synth==0, remov_1harm=0; end % because all synthetic stimuli only have 1 harmonics

        % Group on which to apply function_type
        grpVarFeat_wbf={'sex','d','sr','stim','locationStim','obj',...
                            'stim_SPL','stim_SPL_err','stim_SPL_cat'};

    % Quantile value for the wingbeat frequency and the angular velocity over 1s
        quant=0.75; 

    %%%%%%% End of user parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   


   
   % Variables depends on sex: 
   if strcmp(sex_choice,'f')
        stim_var={'stim_sex','stim_species','stim_soundLevel'};
   else
        stim_var={'stim_sex','stim_species','stim_soundLevel','stim_type','stim_1VSswarm','stim_NumbHarmonics'};%,...
   end
    
    % Function to mean or median the subreplicates to have only 1 value per day and per feature
    function_type={'mean',1,'omitnan'};

    % Condition on number of swarming mosquitoes & sex
    if isnan(Nobjects)
        Nobjects_condition=1;
    else
        Nobjects_condition= any(data_tbl2ter.Nobjects==Nobjects,2) ;
    end
    if strcmp(sex_choice,'f') || strcmp(sex_choice,'m')
        sex_condition=strcmp(data_tbl2ter.sex,sex_choice); 
    else
        sex_condition=1; % 1 for both sexes
    end

    % Table-name setting
    Nobjects_s='';
    for kk=1:length(Nobjects)
       Nobjects_s=strcat(Nobjects_s,num2str(Nobjects(kk))); 
    end


    %%% Computing the features on each {stim,sr} (med(wbf), etc)
    disp('Computing the features on each {stim,sr}...')
    grpVarFeat_wbf( length(grpVarFeat_wbf)+1 : length(grpVarFeat_wbf)+length(stim_var))= stim_var;

    % Row indexes without any sound stimulus
    if length(t_noStim)==2
        cond_noStim=data_tbl2ter.location >= t_noStim(1) & data_tbl2ter.location < t_noStim(2);
    elseif length(t_noStim)==4
        cond_noStim=(data_tbl2ter.location >= t_noStim(1) & data_tbl2ter.location < t_noStim(2)) ...
            | (data_tbl2ter.location >= t_noStim(3) & data_tbl2ter.location < t_noStim(4))  ;
    end


    % Condition on object index
    row=   (cond_noStim ...
        |  (data_tbl2ter.location >= t_stim(1) & data_tbl2ter.location < t_stim(2)))...
        & Nobjects_condition...
        & sex_condition ;

    [G, data_tbl3] = findgroups(data_tbl2ter(row,grpVarFeat_wbf));

    %%% WBF features 
    disp("  Computing wbf features...")
    data_tbl3.wbf_Med=splitapply(@nanmedian, data_tbl2ter(row,'wbf'),G); ;
    data_tbl3.wbf_QuantH=cell2mat(splitapply(@(x){feval('quantile',x,quant)}, data_tbl2ter(row,'wbf'),G)); %
    
    %%% Fligth trajectory feature
    disp("  Computing trajectory features...")
    % Trackit angular velocity
    data_tbl3.avel_Med=splitapply(@nanmedian, data_tbl2ter(row,'avel'),G); % The new variables are packed into 1 single variable array 'temp'        ;
    data_tbl3.avel_QuantH=cell2mat(splitapply(@(x){feval('quantile',x,quant)}, data_tbl2ter(row,'avel'),G)); 

    %%% Computing the diff parameters (between no sound and sound)
    % We add new columns which correspond to diff of all parameters between 1s (or 7s) before the stimulus onset and between 1s (or 7s) after the stimulus onset (ie during the stimulus)
    % This new parameters will be the same in the rows of locationStim=0 and locationStim=1
    disp("  Computing features diff...")    
    row0=data_tbl3.locationStim==0; % rows of data_tbl3  before the stimulus
    row1=data_tbl3.locationStim==1;  % rows of data_tbl3 during the stimulus
    firstCol_parameter_i = 1+length(grpVarFeat_wbf);
    table_col_name= data_tbl3.Properties.VariableNames;
    Ncol_befLoop=size(data_tbl3,2);
    Nrow=size(data_tbl3,1);
    for kk= firstCol_parameter_i:Ncol_befLoop
        newParam_kk_col_i = Ncol_befLoop+1+kk-firstCol_parameter_i;
        newParam_kk = data_tbl3.(kk)(row1) - data_tbl3.(kk)(row0);
        data_tbl3.(newParam_kk_col_i)=NaN(Nrow,1);
        data_tbl3.(newParam_kk_col_i)(row0) =  0; %zeros(sum(row0),1)); % 0 because it is the difference with itself
        data_tbl3.(newParam_kk_col_i)(row1) =  newParam_kk;
        data_tbl3.Properties.VariableNames{newParam_kk_col_i} = char(strcat( data_tbl3.Properties.VariableNames{kk},"_diff" )); % append _diff
    end



    %%% Removing some of the data
    if remov_spl_b
        data_tbl3(data_tbl3.stim_soundLevel=='1b',:)=[]; %remove the 1-female sound stimuli
        data_tbl3(data_tbl3.stim_soundLevel=='2b',:)=[]; %remove the 1-female sound stimuli
        data_tbl3(data_tbl3.stim_soundLevel=='3b',:)=[]; %remove the 1-female sound stimuli
        data_tbl3(data_tbl3.stim_soundLevel=='4b',:)=[]; %remove the 1-female sound stimuli
    end
    if remov_gamb
        data_tbl3(data_tbl3.stim_species=='g',:)=[]; %remove the gambiae sound stimuli
    end
    if ~strcmp(sex_choice,'f')
        if remov_1harm
            data_tbl3(data_tbl3.stim_NumbHarmonics=='1',:)=[]; %remove the males's stimuli with 1 harmonics only
        end
        if remov_synth
            data_tbl3(data_tbl3.stim_type=='s',:)=[]; %remove the males's stimuli with which are synthetic
        end
    end

    %%%%%%% Keeping only 1 value per day (averaging or medianing over the subreplicates)
    disp("Averaging over obj and subreplicates")
    factorToAverageOver={'obj','sr'};
    data_tbl_day=data_tbl3;
    grpVarFeat_temp=grpVarFeat_wbf;
    parameters_name=data_tbl3.Properties.VariableNames(firstCol_parameter_i:end);
    for ftao_ind=1:length(factorToAverageOver)
        % We rename the column of the transformed variables only at the last loop (necessary with the use of fctTableOverFactor in loops):
        if ftao_ind==length(factorToAverageOver) 
            rename=1;
            mean_wbf_name='mean_wbf_Med';
        else
            rename=0;
            mean_wbf_name='wbf_Med';
        end
        % Applying function over factor and reducing the table size accordingly:
        [data_tbl_new,G_temp] = fctTableOverFactor(data_tbl_day,grpVarFeat_temp,parameters_name,factorToAverageOver{ftao_ind},function_type,rename);
        % MEdian: (needed for normlization of other data, so we have to compute them whaterver MeanVariables)
        data_tbl_new.(mean_wbf_name) = cell2mat(...
            splitapply(@(x){feval(function_type{1},x,function_type{2},function_type{3})},...
                       data_tbl_day.wbf_Med,G_temp));
        % Loop update
        data_tbl_day=data_tbl_new;
        grpVarFeat_temp(ismember(grpVarFeat_temp,factorToAverageOver{ftao_ind}))=[]; % udate of the column list while we have just removed one       
    end

    if length(t_noStim)==2
        title_noStim=strcat('_Tns=',num2str(t_noStim(1)),'-',num2str(t_noStim(2)-t_step));
    elseif length(t_noStim)==4
        title_noStim=strcat('_Tns=',num2str(t_noStim(1)),'-',num2str(t_noStim(2)-t_step),'and',num2str(t_noStim(3)),'-',num2str(t_noStim(4)-t_step));
    end

    %%% Generic title
    save_commun=strcat('sex=',sex_choice,... '_step=', num2str(t_step),'s',...
                '_removeGamb', num2str(remov_gamb),...
                '_removeSplB', num2str(remov_spl_b),...
                title_noStim,...
                '_Ts=',num2str(t_stim(1)),'-',num2str(t_stim(2)-t_step),...
                '_Nobj=',Nobjects_s,...
                '_quant=',num2str(quant));
            
            
    %%% Writing the table in a text file
    writetable(data_tbl_day,strcat('data_tbl_',...
                                   save_commun,...
                                   '_',date,file_name_add,'.txt'),...
                'Delimiter',',');  

end