function y = averagingOverEachObject(VidData_stim_o_t_x,wind_t_sec,t_start,t_step,t_end)
%Aveage over a greater sample period and then averaged over each object
%
% Arguments
%   VidData_stim_o_t_x  3 columns:  
%                           VidData_stim(:,1) object numbers
%                           VidData_stim(:,2) time (sec)
%                           VidData_stim(:,feature) Feature
%   wind_t_sec          Initial Sampling period
%   t_start            time to start the averaging (sec)
%   t_step             New sampling period
%   t_end            time to end the averaging (sec)
%
% 18/01/2021: bug correction, particularly in case of t_step=wind_t_sec which was doing some averaging over time

if wind_t_sec > t_step
    error('New sampling period (t_step) should be greater than former sampling period (wind_t_sec)')
end
if size(VidData_stim_o_t_x,2)~=3
    error('VidData_stim_o_t_x should have 3 coluhmns')
end

%%% Increasing sample period (and averaging over the previous sample period)
object_v=unique( VidData_stim_o_t_x(:,1) )';
object_ind_min=0;
Z=[];
for object_ind=object_v
    object_ind_min=object_ind_min+1;
    row= VidData_stim_o_t_x(:,1)==object_ind ;
    Z_newFs=VidData_stim_o_t_x( row,3); % Z at fs_trackit for 1 object
    index=0;
    for tZ_i=t_start:t_step:min(t_end,max(VidData_stim_o_t_x(:,2))) % mean over the new time window t_step
        index=index+1;
        time_v=tZ_i:wind_t_sec:tZ_i+t_step/3; % time window
        Z(index,object_ind_min) = nanmean(   Z_newFs( ismembertol(VidData_stim_o_t_x(row,2),time_v,t_step/3)   ));
    end
end
%%% Averaging over the objects
y= nanmean(Z,2);  % mean over the objects 

end

