function [data_tbl_new,G] = fctTableOverFactor(data_tbl,grpVar,var_toBeTransformed,factorToAverageOver,function_type,rename)
% Apply function (i.e. averaging) over specifical table column (a particlar column of a table
% Innputs:
%   data_tbl                Data table
%   grpVar                  Table column name to keep
%   var_toBeTransformed        Variables to apply function_type
%   factorToAverageOver     Name of the data_tbl3 column to average over
%   function_type           Function to apply on group
%   rename                  Boolean. To rename or not the column which values have changed
% Outputs
%   data_tbl_new            Table with only grpVar columns minus factorToAverageOver column. All the values were transformed according to function function_type over factorToAverageOver column
%   G                       table GRoup over grpVar minus factorToAverageOver

    % New Name of colum (for variable after having being transformed)
    if rename
        MeanVariables=[];
        for var_toBeAnalized_ind=1:length(var_toBeTransformed)
            MeanVariables{var_toBeAnalized_ind}=strcat(function_type{1},'_',char(var_toBeTransformed{var_toBeAnalized_ind})) ; %,'meanTemper30cmHigh','meanTemper85cmHigh'};
        end
    else
        MeanVariables=var_toBeTransformed;
    end

     % We remove the 'sr' element:
    grpVar(ismember(grpVar,factorToAverageOver))=[];
    % We compte the groups for each day
    [G, data_tbl_new] = findgroups(data_tbl(:,grpVar)) ;
    % The mean or median are aplied over all the subreplicates in order tto have 1 value per {grpVarFeat_wbf_day} and are added to the new table
    for MeanVariables_ind=1:length(MeanVariables)
        data_tbl_new.(MeanVariables{MeanVariables_ind}) = cell2mat(...
            splitapply(@(x){feval(function_type{1},x,function_type{2},function_type{3})},...
                       data_tbl.(var_toBeTransformed{MeanVariables_ind}),G)); 
    end

end